#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef QWS
#  include <qpe/qpeapplication.h>
#else
#  include <qapplication.h>
#endif

#include <qwidget.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qpainter.h>
#include <qcolor.h>
#include <qmainwindow.h>

#include "mainwindow.h"

extern "C" {
#include "fb.h"
#include "rc.h"
}

// Argv and argc hack to fool QApplication
static char * argv[1] = { "qtopiagnuboy" };
static int argc = 1;
struct fb fb;

// Set the scalling here. Just be aware that scalling takes a lot of CPU!
// Use a scale of 1.5 to fill the entire PDA screen.
static float scale = 1;


#ifdef QWS
static QPEApplication qtopia_app(argc, argv);
#else
static QApplication qtopia_app(argc, argv);
#endif

static QImage qtopia_image(160, 144, 32, 0, QImage::LittleEndian);
static MainWindow qtopia_window(NULL);
static QPainter qtopia_display_painter;
static QWidget *qtopia_display;


rcvar_t vid_exports[] = {
    RCV_END
};

rcvar_t joy_exports[] = {
    RCV_END
};

extern "C" {

void vid_resize()
{
    printf("vid_resize\n");
}

void vid_preinit()
{
    printf("vid_preinit\n");
    
    qtopia_display = qtopia_window.displayWidget();
    qtopia_app.setMainWidget(&qtopia_window);
    
#ifdef QWS
    qtopia_window.showMaximized();
#else
    qtopia_window.setGeometry(0, 0, 240, 320);
    qtopia_window.show();
#endif
}

void vid_init()
{
    printf("vid_init\n");
    
    int red_length, green_length, blue_length;
	int red_offset, green_offset, blue_offset;

	fb.w = qtopia_image.width();
	fb.h = qtopia_image.height();
	fb.pitch = qtopia_image.bytesPerLine();
	fb.ptr = qtopia_image.bits();

	fb.pelsize = qtopia_image.depth() / 8;
	fb.indexed = (qtopia_image.depth() == 8) ? 1 : 0;

    // Magic 'make work' code copied from thinlib
	switch (qtopia_image.depth())
	{
	case 8:
		red_length = 0;
		green_length = 0;
		blue_length = 0;
		red_offset = 0;
		green_offset = 0;
		blue_offset = 0;
		break;

	case 16:
		red_length = 5;
		green_length = 6;
		blue_length = 5;
		red_offset = 11;
		green_offset = 5;
		blue_offset = 0;
		break;

	case 32:
		red_length = 8;
		green_length = 8;
		blue_length = 8;
		red_offset = 16;
		green_offset = 8;
		blue_offset = 0;
		break;

	case 15:
	case 24:
	default:
		printf("i don't know what to do with %d bpp mode", 
               qtopia_image.depth());
        abort();
		break;
	}

	fb.cc[0].r = 8 - red_length;
	fb.cc[1].r = 8 - green_length;
	fb.cc[2].r = 8 - blue_length;
	fb.cc[0].l = red_offset;
	fb.cc[1].l = green_offset;
	fb.cc[2].l = blue_offset;

	fb.enabled = 1;
	fb.dirty = 0;
}

void vid_close()
{
    printf("vid_close\n");
    
    // if the main window is still showing, hide it
    if (qtopia_window.isVisible())
        qtopia_window.hide();
}

void vid_end()
{
    //printf("vid_end\n");

    int x = 0;
    int y = 0;
    QImage image;
    
    if (scale != 1)
    {
        int newWidth = (int)(qtopia_image.width() * scale);
        int newHeight = (int)(qtopia_image.height() * scale);
        
        image = qtopia_image.smoothScale(newWidth, newHeight);
    }
    else
    {
        // This shouldn't cost anything since Qt does shared memory
        image = qtopia_image;
    }
    
    // Center the image
    x = qtopia_display->width() / 2;
    x = x - image.width() / 2;
    
    y = qtopia_display->height() / 2;
    y = y - image.height() / 2;
    
    qtopia_display_painter.begin(qtopia_display);
    qtopia_display_painter.drawImage(x, y, image);    
    qtopia_display_painter.end();
}

void vid_settitle(char  *title)
{
    printf("vid_settitle\n");
    
    QString caption("Qtopia GnuBoy: ");
    caption += title;
    
    qtopia_window.setCaption(caption);
}

void vid_setpal(int i, int r, int g, int b)
{
    printf("vid_setpal\n");
  
  // This doesn't seem to actually work. Maybe someone who understands
  // graphics and fix this. -mike
    qtopia_image.setColor(i, qRgb(r, g, b));
}

void vid_begin()
{
    //printf("vid_begin\n");
}

void ev_poll()
{
    //printf("ev_poll");
    
    static bool skipThisEvent = true;
    
    // QApp will process multiple events at the same time, so it
    // should be ok to skip every other call to process events.
    // Hopefully this will buy us some speed.
    
    if (!skipThisEvent)
        qApp->processEvents();
        
    skipThisEvent = !skipThisEvent;
 
}

}
