#include "qtopiagnuboyl.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qstring.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qdir.h>
#include <qpe/config.h>
#include <qpe/process.h>

QtopiaGnuboyL::QtopiaGnuboyL(QWidget *parent, const char *name)
    : QMainWindow(parent, name), fDirLineEdit(NULL), fRomListBox(NULL),
      fUpdateButton(NULL), fLaunchButton(NULL)
{
    setCaption(tr("Qtopia Gnuboy Launcher v") + VERSION);
    
    initGUI();
    connectSlots();
    initialize();
}

QtopiaGnuboyL::~QtopiaGnuboyL()
{
    // Save the last dir
    Config c("QtopiaGnuboyL");
    
    c.setGroup("history");
    c.writeEntry("lastDir", fDirLineEdit->text());
}

void QtopiaGnuboyL::initGUI()
{
    QWidget *cw = new QWidget(this, "cw");
    setCentralWidget(cw);
    
    QVBoxLayout *layout = new QVBoxLayout(cw);
    layout->setSpacing(2);
    layout->setMargin(3);
    
    QString instruct = tr("Enter the directory containing the gameboy roms:");
    QLabel *label = new QLabel(instruct, cw);
    label->setAlignment(QLabel::AlignTop | QLabel::AlignLeft |
                        QLabel::WordBreak);
    layout->addWidget(label);
    
    QWidget *dirBox = new QWidget(cw);
    QHBoxLayout *dirBoxLayout = new QHBoxLayout(dirBox);
    layout->addWidget(dirBox);
    
    fDirLineEdit = new QLineEdit("/", dirBox, "fDirLineEdit");
    dirBoxLayout->addWidget(fDirLineEdit);
    dirBoxLayout->setStretchFactor(fDirLineEdit, 1);
    
    fUpdateButton = new QPushButton(tr("Apply"), dirBox, "fUpdateButton");
    dirBoxLayout->addWidget(fUpdateButton);
    
    QFrame *hLine = new QFrame(cw);
    hLine->setFrameStyle(QFrame::HLine | QFrame::Sunken);
    layout->addWidget(hLine);
    
    label = new QLabel(tr("Roms Found:"), cw);
    layout->addWidget(label);
    
    fRomListBox = new QListBox(cw, "fRomListBox");
    layout->addWidget(fRomListBox);
    
    fLaunchButton = new QPushButton(tr("Launch Emulator"), cw,
                                    "fLaunchButton");
    layout->addWidget(fLaunchButton);
}

void QtopiaGnuboyL::connectSlots()
{
    connect(fUpdateButton, SIGNAL(clicked()), this,
        SLOT(slotUpdateButtonClicked()));
    connect(fDirLineEdit, SIGNAL(returnPressed()), this,
        SLOT(slotUpdateButtonClicked()));
    connect(fLaunchButton, SIGNAL(clicked()), this,
        SLOT(slotLaunchButtonClicked()));
    connect(fRomListBox, SIGNAL(currentChanged(QListBoxItem*)), this,
        SLOT(slotRomCurrentChanged(QListBoxItem *)));
}

void QtopiaGnuboyL::initialize()
{
     Config c("QtopiaGnuboyL");
    
    c.setGroup("history");
    QString path = c.readEntry("lastDir", QDir::homeDirPath());
    
    // Initialize to the users home directory or last saved
    fDirLineEdit->setText(path);
    
    slotUpdateButtonClicked();
}

//////////////////////////////
// SLOTS

void QtopiaGnuboyL::slotUpdateButtonClicked()
{
    QString path = fDirLineEdit->text();
    
    // Clear the list box
    fRomListBox->clear();
    fLaunchButton->setEnabled(false);
    
    // Make sure it is a dir
    QDir dir(path);
    if (!dir.exists())
        return;
        
    // Get a list of all the roms
    QStringList items = dir.entryList("*.gbc;*.gb;*.GB;*.GBC", 
                                      QDir::Files | QDir::Readable);
                                      
    fRomListBox->insertStringList(items);
}

void QtopiaGnuboyL::slotLaunchButtonClicked()
{
    // If we have a rom, run gnuboy
    QString rom = fRomListBox->currentText();
    
    if (rom == QString::null)
        return;
        
    rom = fDirLineEdit->text() + QString("/") + rom;
    
    QStringList args;
    args << "qtopiagnuboy" << rom;
    QString in;
    QString out;
    Process *p = new Process(args, this);
    p->exec(in, out, NULL);
}

void QtopiaGnuboyL::slotRomCurrentChanged(QListBoxItem *item)
{
    fLaunchButton->setEnabled(item != NULL);
}
